//=================================================================================================
//
// Company:       Paul Scherrer Institut
//                5232 Villigen PSI
//                Switzerland
//
//-------------------------------------------------------------------------------------------------
//
// Project:       Peltier Controller V2
// Author:        Noah Piqué (noah.pique@psi.ch)
//
//-------------------------------------------------------------------------------------------------
//
// Module:        Variable Handler
// Filename:      VARH_VariableHandler.h
// Date:          Handled by Subversion (version control system)
// Revision:      Handled by Subversion (version control system)
// History:       Handled by Subversion (version control system)
//
//-------------------------------------------------------------------------------------------------

#ifndef VARH_VARIABLEHANDLER_H
#define VARH_VARIABLEHANDLER_H

#ifdef __cplusplus
extern "C" {
#endif



//=================================================================================================
// Section:       INCLUDES
// Description:   List of required include files (visible by all modules).
//=================================================================================================

#include "../SDEF_StandardDefinitions.h"


//=================================================================================================
// Section:       DEFINITIONS
// Description:   Definition of global constants (visible by all modules).
//=================================================================================================

// Flags
#define VARH_FLAG_NONE      0       // no flag
#define VARH_FLAG_FLOAT     (1<<0)  // channel in floating point format
#define VARH_FLAG_SIGNED    (1<<1)  // channel is signed integer
#define VARH_FLAG_DATALESS  (1<<2)  // channel doesn't contain data; displayed on the host, but not writable
#define VARH_FLAG_HIDDEN    (1<<3)  // used for internal config parameters; not displayed on the host pc, but writable
#define VARH_FLAG_REMIN     (1<<4)  // get variable from remote node on subbus
#define VARH_FLAG_REMOUT    (1<<5)  // send variable to remote node on subbus
#define VARH_FLAG_READONLY  (1<<7)  // variable is readonly, host can not set variable

#define VARH_HV_MAX         (300.0f)  
#define VARH_I_MAX          (1000.0f)

//=================================================================================================
// Section:       MACROS
// Description:   Definition of global macros (visible by all modules).
//=================================================================================================



//=================================================================================================
// Section:       ENUMERATIONS
// Description:   Definition of global enumerations (visible by all modules).
//=================================================================================================

// Units
typedef enum
{
  // physical units
  VARH_eUnitUndefined   = 0,    // 00 Undefined unit
  VARH_eUnitMeter       = 1,    // 01 Meter
  VARH_eUnitGram        = 2,    // 02 Gram
  VARH_eUnitSecond      = 3,    // 03 Second
  VARH_eUnitMinute      = 4,    // 04 Minute
  VARH_eUnitHour        = 5,    // 05 Hour
  VARH_eUnitAmpere      = 6,    // 06 Ampere
  VARH_eUnitKelvin      = 7,    // 07 Kelvin
  VARH_eUnitCelsius     = 8,    // 08 Celsius
  VARH_eUnitFarenheit   = 9,    // 09 Farenheit
  
  // SI base units
  VARH_eUnitHertz       = 20,   // 20 Hertz
  VARH_eUnitPascal      = 21,   // 21 Pascal
  VARH_eUnitBar         = 22,   // 22 Bar
  VARH_eUnitWatt        = 23,   // 23 Watt
  VARH_eUnitVolt        = 24,   // 24 Volt
  VARH_eUnitOhm         = 25,   // 25 Ohm
  VARH_eUnitTesla       = 26,   // 26 Tesla
  VARH_eUnitLiterPerSec = 27,   // 27 Liter per second
  VARH_eUnitRPM         = 28,   // 28 Rounds per minute
  VARH_eUnitFarad       = 29,   // 29 Farad
  VARH_eUnitJoule       = 30,   // 30 Joule
  
  // computer units
  VARH_eUnitBoolean     = 50,   // 50 Boolean
  VARH_eUnitByte        = 52,   // 52 Byte
  VARH_eUnitWord        = 53,   // 53 Word (16Bit)
  VARH_eUnitDoubleWord  = 54,   // 54 Double Word (32Bit)
  VARH_eUnitASCII       = 55,   // 55 ASCII character
  VARH_eUnitString      = 56,   // 56 String
  VARH_eUnitBaud        = 57,   // 57 Baud
  
  // others
  VARH_eUnitPercent     = 90,   // 90 Percent
  VARH_eUnitPPM         = 91,   // 91 Parts per million
  VARH_eUnitCount       = 92,   // 92 Count
  VARH_eUnitFactor      = 93,   // 93 Factor
  
} VARH_EnUnit;

// Prefix
typedef enum
{
  VARH_ePrefixPico        = -12,
  VARH_ePrefixNano        = -9,
  VARH_ePrefixMicro       = -6,
  VARH_ePrefixMilli       = -3,
  VARH_ePrefixNone        = 0,
  VARH_ePrefixKilo        = 3,
  VARH_ePrefixMega        = 6,
  VARH_ePrefixGiga        = 9,
  VARH_ePrefixTera        = 12,
} VARH_EnPrefix;


typedef enum
{
  VARH_eWrite        = 0,
  VARH_eNewValue,
  VARH_eRising,
  VARH_eFalling,
  
  VARH_eNumberOfNotificationTypes,
} VARH_EnNotification;


// variables
typedef enum
{ 
	VARH_eStatus,
	
  VARH_eVoltageRefEn,
  
  VARH_eStatusG,
  VARH_eStatusR,
  VARH_eEnabledG,
  VARH_eEnabledR,
  VARH_eInvertedG,
  VARH_eInvertedR,
  VARH_eSpareG,
  VARH_eSpareR,
 
  VARH_eTempHeatSink,
  VARH_eTempShunt,
  VARH_eTempVoltageRef,
	VARH_eTempCurrentSrc, 

  VARH_eNumberOfVariables,   // Must be last entry     
} VARH_EnVariables;



//=================================================================================================
// Section:       STRUCTURES
// Description:   Definition of global Structures (visible by all modules).
//=================================================================================================

#pragma pack(4)
typedef union {
   FLOAT  flVal;
   U32    u32Val;
   S32    s32Val; 
} VARH_UVariable; 
#pragma pack()

typedef struct
{
  CHAR                      acName[16];             // name
  U8                        u8Flags;                // flags
  VARH_EnUnit               eUnit;                  // unit
  VARH_EnPrefix             ePrefix;                // prefix
  VARH_UVariable            uInitData;              // initial Data (data is always 32 bit, at least internal)
} VARH_StVarInfo;


//=================================================================================================
// Section:       GLOBAL VARIABLES
// Description:   Definition of global variables (visible by all modules).
//=================================================================================================



//=================================================================================================
// Section:       GLOBAL CONSTANTS
// Description:   Definition of global constants (visible by all modules).
//=================================================================================================


//=================================================================================================
// Section:       FUNCTION TYPES
// Description:   Definition of functions
//=================================================================================================
typedef VOID (*VARH_pfnNotification)( PVOID pvData );
typedef BOOL (*VARH_pfnRangeCheck)( PVOID pvCallbackData, VARH_UVariable uNewValue );

//=================================================================================================
// Section:       GLOBAL FUNCTIONS (PROTOTYPES)
// Description:   Definition of global functions (visible by all modules).
//=================================================================================================

BOOL VARH_boInitializeModule( BOOL boInitConfig );
BOOL VARH_boRegisterNotification( U8 u8Variable, VARH_EnNotification enNotification, VARH_pfnNotification pfnCallback, PVOID pvCallbackArgument );
BOOL VARH_boRegisterRangeCheck( U8 u8Variable, VARH_pfnRangeCheck pfnCallback, PVOID pvCallbackArgument ); 

// set data functions
VOID VARH_vSetVariableData( U8 u8Variable, VARH_UVariable uData );
VOID VARH_vSetBit( U8 u8Variable, U32 u32BitMask );
U32  VARH_u32GetBit( U8 u8Variable,  U32 u32BitMask );
VOID VARH_vClearBit( U8 u8Variable, U32 u32BitMask );
VOID VARH_vIncrement( U8 u8Variable );
VOID VARH_vSetVariableDataFromClient( U8 u8Variable, VARH_UVariable uData );

// get data function
VARH_UVariable VARH_uGetVariableData( U8 u8Variable );
VARH_UVariable VARH_u32GetVariableDataFromClient( U8 u8Variable );

// check functions
BOOL VARH_boBooleanVarCheck( PVOID pvCallbackData, VARH_UVariable uNewValue );

CONST VARH_StVarInfo* VARH_pstGetVariableInfo( U8 u8Variable );
VOID VARH_vSetVariableToInitData( U8 u8Variable );
VOID VARH_vSetAllVariablesToInitData( VOID );

U8 VARH_u8GetNumberOfVariables( VOID );

#ifdef __cplusplus
}
#endif

#endif
